@@ -2,6 +2,8 @@
 
 #include "test-utils.h"
 
+#ifdef HAVE_APACHE
+
 static GMainLoop *loop;
 
 typedef struct {
@@ -1201,3 +1203,13 @@ main (int argc, char **argv)
 	test_cleanup ();
 	return errors != 0;
 }
+
+#else /* HAVE_APACHE */
+
+int
+main (int argc, char **argv)
+{
+	return 77; /* SKIP */
+}
+
+#endif
